@@ -60,37 +60,81 @@ from numpy import newaxis
 from scipy.special import expit, logit
 
 from . import lattice, matrix as gtmatrix
+from .basis import pole
 from ._version import get_versions
 
 __version__ = get_versions()['version']
 
 LOGGER = logging.getLogger(__name__)
 
+# Bethe lattice
+# pylint: disable=wrong-import-position
+from .lattice.bethe import (dos as bethe_dos,
+                            dos_moment as bethe_dos_moment,
+                            gf_d1_z as bethe_gf_d1_z,
+                            gf_d2_z as bethe_gf_d2_z,
+                            gf_z as bethe_gf_z,
+                            hilbert_transform as bethe_hilbert_transform)
+
+# silence warnings of unused imports
+assert (bethe_dos and bethe_dos_moment and bethe_gf_d1_z and bethe_gf_d2_z
+        and bethe_gf_z and bethe_hilbert_transform)
+
+bethe_dos.m1 = partial(lattice.bethe.dos_moment, 1)
+bethe_dos.m2 = partial(lattice.bethe.dos_moment, 2)
+bethe_dos.m3 = partial(lattice.bethe.dos_moment, 3)
+bethe_dos.m4 = partial(lattice.bethe.dos_moment, 4)
+bethe_dos.m5 = partial(lattice.bethe.dos_moment, 5)
+
+
+# One-dimensional lattice
+from .lattice.onedim import (dos as onedim_dos,
+                             dos_moment as onedim_dos_moment,
+                             gf_z as onedim_gf_z,
+                             hilbert_transform as onedim_hilbert_transform)
+
+# silence warnings of unused imports
+assert (onedim_dos and onedim_dos_moment and onedim_gf_z and onedim_hilbert_transform)
+
+onedim_dos.m1 = partial(lattice.onedim.dos_moment, 1)
+onedim_dos.m2 = partial(lattice.onedim.dos_moment, 2)
+onedim_dos.m3 = partial(lattice.onedim.dos_moment, 3)
+onedim_dos.m4 = partial(lattice.onedim.dos_moment, 4)
+onedim_dos.m5 = partial(lattice.onedim.dos_moment, 5)
+
+
+# Square lattice
+from .lattice.square import (dos as square_dos,
+                             dos_moment as square_dos_moment,
+                             gf_z as square_gf_z,
+                             hilbert_transform as square_hilbert_transform)
+
+# silence warnings of unused imports
+assert (square_dos and square_dos_moment and square_gf_z and square_hilbert_transform)
+
+square_dos.m1 = partial(lattice.square.dos_moment, 1)
+square_dos.m2 = partial(lattice.square.dos_moment, 2)
+square_dos.m3 = partial(lattice.square.dos_moment, 3)
+square_dos.m4 = partial(lattice.square.dos_moment, 4)
+square_dos.m5 = partial(lattice.square.dos_moment, 5)
+
 
 def bose_fct(eps, beta):
     r"""Return the Bose function `1/(exp(βϵ)-1)`.
 
     Parameters
     ----------
-    eps : complex or float or ndarray
+    eps : complex or float or array_like
         The energy at which the Bose function is evaluated.
-        The real part needs to be positive `eps.real > 0`
     beta : float
         The inverse temperature :math:`beta = 1/k_B T`.
 
     Returns
     -------
-    bose_fct : complex of float or ndarray
+    bose_fct : complex of float np.ndarray
         The Bose function, same type as eps.
 
-    Raises
-    ------
-    ValueError
-        If `eps.real < 0`.
-
     """
-    if np.any(eps.real < 0):
-        raise ValueError("Bose function only well defined for non-negative energies `eps`.")
     betaeps = np.asanyarray(beta*eps)
     res = np.empty_like(betaeps)
     small = betaeps < 700
@@ -239,27 +283,56 @@ def matsubara_frequencies_b(n_points, beta):
     return 2j * np.pi / beta * n_points
 
 
-# Bethe lattice
-bethe_gf_z = lattice.bethe.gf_z
-bethe_gf_d1_z = lattice.bethe.gf_d1_z
-bethe_gf_d2_z = lattice.bethe.gf_d2_z
-bethe_hilbert_transfrom = lattice.bethe.hilbert_transform
-bethe_dos = lattice.bethe.dos
-bethe_dos.m1 = partial(lattice.bethe.dos_moment, 1)
-bethe_dos.m2 = partial(lattice.bethe.dos_moment, 2)
-bethe_dos.m3 = partial(lattice.bethe.dos_moment, 3)
-bethe_dos.m4 = partial(lattice.bethe.dos_moment, 4)
-bethe_dos.m5 = partial(lattice.bethe.dos_moment, 5)
+def pade_frequencies(num: int, beta):
+    """Return `num` *fermionic* Padé frequencies :math:`iz_p`.
 
-# Square lattice
-square_gf_z = lattice.square.gf_z
-square_hilbert_transfrom = lattice.square.hilbert_transform
-square_dos = lattice.square.dos
-square_dos.m1 = partial(lattice.square.dos_moment, 1)
-square_dos.m2 = partial(lattice.square.dos_moment, 2)
-square_dos.m3 = partial(lattice.square.dos_moment, 3)
-square_dos.m4 = partial(lattice.square.dos_moment, 4)
-square_dos.m5 = partial(lattice.square.dos_moment, 5)
+    The Padé frequencies are the poles of the approximation of the Fermi
+    function with `2*num` poles [ozaki2007]_.
+    This gives an non-equidistant mesh on the imaginary axis.
+
+    Parameters
+    ----------
+    num : int
+        Number of positive Padé frequencies.
+    beta : float
+        The inverse temperature :math:`beta = 1/k_B T`.
+
+    Returns
+    -------
+    izp : (num) complex np.ndarray
+        Positive Padé frequencies.
+    resids : (num) float np.ndarray
+        Residue of the Fermi function corresponding to `izp`. The residue is
+        given relative to true residue of the Fermi function corresponding to
+        the poles at Matsubara frequencies. This allows to use Padé frequencies
+        as drop-in replacement.
+        The actual residues would be `-resids/beta`.
+
+    References
+    ----------
+    .. [ozaki2007] Ozaki, Taisuke. Continued Fraction Representation of the
+       Fermi-Dirac Function for Large-Scale Electronic Structure Calculations.
+       Physical Review B 75, no. 3 (January 23, 2007): 035123.
+       https://doi.org/10.1103/PhysRevB.75.035123.
+
+    .. [hu2010] J. Hu, R.-X. Xu, and Y. Yan, “Communication: Padé spectrum
+       decomposition of Fermi function and Bose function,” J. Chem. Phys., vol.
+       133, no. 10, p.  101106, Sep. 2010, https://doi.org/10.1063/1.3484491
+
+    """
+    num = 2*num
+    a = -np.diagflat(range(1, 2*num, 2))
+    b = np.zeros_like(a, dtype=np.float_)
+    np.fill_diagonal(b[1:, :], 0.5)
+    np.fill_diagonal(b[:, 1:], 0.5)
+    eig, v = sp.linalg.eig(a, b=b, overwrite_a=True, overwrite_b=True)
+    sort = np.argsort(eig)
+    izp = 1j/beta * eig[sort]
+    resids = (0.25*v[0]*np.linalg.inv(v)[:, 0]*eig**2)[sort]
+    assert np.allclose(-izp[:num//2][::-1], izp[num//2:])
+    assert np.allclose(resids[:num//2][::-1], resids[num//2:])
+    assert np.all(~np.iscomplex(resids))
+    return izp[num//2:], resids.real[num//2:]
 
 
 def surface_gf_zeps(z, eps, hopping_nn):
@@ -371,71 +444,30 @@ def hubbard_I_self_z(z, U, occ):
     Σ_{Hub I} : complex array_like
         The self-energy in Hubbard I approximation.
 
+    Examples
+    --------
+    >>> U = 5
+    >>> mu = U/2  # particle-hole symmetric case -> n=0.5
+    >>> ww = np.linspace(-5, 5, num=1000) + 1e-6j
+
+    >>> self_ww = gt.hubbard_I_self_z(ww+mu, U, occ=0.5)
+
+    Show the spectral function for the Bethe lattice,
+    we see the two Hubbard bands centered at ±U/2:
+
+    >>> import matplotlib.pyplot as plt
+    >>> gf_iw = gt.bethe_gf_z(ww+mu-self_ww, half_bandwidth=1.)
+    >>> __ = plt.plot(ww.real, -1./np.pi*gf_iw.imag)
+    >>> plt.show()
+
     """
     hartree = U * occ
     return hartree * z / (z - U + hartree)
 
 
-def pole_gf_z(z, poles, weights):
-    """Green's function given by a finite number of `poles`.
-
-    To be a Green's function, `np.sum(weights)` has to be 1 for the 1/z tail.
-
-    Parameters
-    ----------
-    z : (...) complex array_like
-        Green's function is evaluated at complex frequency `z`.
-    poles, weights : (..., N) float array_like or float
-        The position and weight of the poles.
-
-    Returns
-    -------
-    pole_gf_z : (...) complex np.ndarray
-        Green's function.
-
-    See Also
-    --------
-    pole_gf_tau : corresponding fermionic imaginary time Green's function
-    pole_gf_tau_b : corresponding bosonic imaginary time Green's function
-
-    """
-    poles = np.atleast_1d(poles)
-    z = np.asanyarray(z)[..., newaxis]
-    return np.sum(weights/(z-poles), axis=-1)
-
-
-def pole_gf_tau(tau, poles, weights, beta):
-    """Imaginary time Green's function given by a finite number of `poles`.
-
-    Parameters
-    ----------
-    tau : (...) float array_like
-        Green's function is evaluated at imaginary times `tau`.
-        Only implemented for :math:`τ ∈ [0, β]`.
-    poles, weights : (..., N) float array_like or float
-        Position and weight of the poles.
-    beta : float
-        Inverse temperature
-
-    Returns
-    -------
-    pole_gf_tau : (...) float np.ndarray
-        Imaginary time Green's function.
-
-    See Also
-    --------
-    pole_gf_z : corresponding commutator Green's function
-
-    """
-    assert np.all((tau >= 0.) & (tau <= beta))
-    poles = np.atleast_1d(poles)
-    tau = np.asanyarray(tau)[..., newaxis]
-    beta = np.asanyarray(beta)[..., newaxis]
-    # exp(-tau*pole)*f(-pole, beta) = exp((beta-tau)*pole)*f(pole, beta)
-    exponent = np.where(poles.real >= 0, -tau, beta - tau) * poles
-    # -(1-gt.fermi_fct(poles, beta=beta))*np.exp(-tau*poles)
-    single_pole_tau = np.exp(exponent) * fermi_fct(-np.sign(poles.real)*poles, beta)
-    return -np.sum(weights*single_pole_tau, axis=-1)
+pole_gf_z = pole.gf_z
+pole_gf_d1_z = pole.gf_d1_z
+pole_gf_tau = pole.gf_tau
 
 
 def pole_gf_tau_b(tau, poles, weights, beta):
@@ -496,29 +528,7 @@ def pole_gf_tau_b(tau, poles, weights, beta):
     return np.sum(weights*bose_fct(poles, -beta)*np.exp(-tau*poles), axis=-1)
 
 
-def pole_gf_moments(poles, weights, order):
-    r"""High-frequency moments of the pole Green's function.
-
-    Return the moments `mom` of the expansion :math:`g(z) = \sum_m mom_m/z^m`
-    For the pole Green's function we have the simple relation
-    :math:`1/(z - ϵ) = \sum_{m=1} ϵ^{m-1}/z^m`.
-
-    Parameters
-    ----------
-    poles, weights : (..., N) float np.ndarray
-        Position and weight of the poles.
-    order : (..., M) int array_like
-        Order (degree) of the moments.
-
-    Returns
-    -------
-    mom : (..., M) float np.ndarray
-        High-frequency moments.
-
-    """
-    poles, weights = np.atleast_1d(*np.broadcast_arrays(poles, weights))
-    order = np.asarray(order)[..., newaxis]
-    return np.sum(weights[..., newaxis, :] * poles[..., newaxis, :]**(order-1), axis=-1)
+pole_gf_moments = pole.moments
 
 
 def chemical_potential(occ_root: Callable[[float], float], mu0=0.0, step0=1.0, **kwds) -> float:
@@ -704,7 +714,8 @@ def density(gf_iw, potential, beta, return_err=True, matrix=False, total=False):
     if matrix:
         dec = gtmatrix.decompose_hamiltonian(potential)
         xi = dec.xi
-        tail = dec.reconstruct(1./np.add.outer(xi, iw), kind='diag')
+        tail = dec.reconstruct(1./np.add.outer(iw, xi), kind='diag')
+        tail = np.moveaxis(tail, source=0, destination=-1)
         analytic = dec.reconstruct(fermi_fct(-xi, beta=beta), kind='diag')
     else:
         tail = 1/np.add.outer(potential, iw)
@@ -729,6 +740,92 @@ def density(gf_iw, potential, beta, return_err=True, matrix=False, total=False):
     return density
 
 
+def density_iw(iws, gf_iw, beta, weights=1., moments=(1.,), n_fit=0):
+    r"""Calculate the number density of the Green's function `gf_iw` at finite temperature `beta`.
+
+    This function can be used for fermionic Matsubara frequencies `matsubara_frequencies`,
+    as well as fermionic Padé frequencies `pade_frequencies`.
+
+    Parameters
+    ----------
+    iws, gf_iw : (..., N_iw) complex np.ndarray
+        Positive Matsubara frequencies :math:`iω_n` (or Padé :math:`iz_p`)
+        and the Green's function at these frequencies.
+    beta : float
+        The inverse temperature :math:`beta = 1/k_B T`.
+    weights : (..., N_iw) float np.ndarray, optional
+        Residues of the frequencies with respect to the residues of the
+        Matsubara frequencies `1/beta`. (default: 1.)
+        For Padé frequencies this needs to be provided.
+    moments : (..., M) float array_like, optional
+        Moments of the high-frequency expansion, where
+        `G(z) = moments / z**np.arange(N)` for large `z`.
+    n_fit : int, optional
+        Number of additionally to `moments` fitted moments. If Padé frequencies
+        are used, this is typically not necessary. (default: 0)
+
+    Returns
+    -------
+    occ : float
+        The number density of the given Green's function `gf_iw`.
+
+    See Also
+    --------
+    matsubara_frequencies : Method generating Matsubara frequencies `iws`.
+    pade_frequencies : Method generating Padé frequencies `iws` with `weights`.
+
+    Examples
+    --------
+    >>> BETA = 50
+    >>> iws = gt.matsubara_frequencies(range(1024), beta=BETA)
+
+    Example Green's function
+
+    >>> np.random.seed(0)  # to have deterministic results
+    >>> poles = 2*np.random.random(10) - 1  # partially filled
+    >>> residues = np.random.random(10); residues = residues / np.sum(residues)
+    >>> pole_gf = gt.basis.PoleGf(poles=poles, residues=residues)
+    >>> gf_iw = pole_gf.eval_z(iws)
+    >>> exact = pole_gf.occ(BETA)
+    >>> exact
+    0.17858151698239388
+
+    Numerical calculation of the occupation number,
+    using Matsubara frequency
+
+    >>> occ = gt.density_iw(iws, gf_iw, beta=BETA)
+    >>> m2 = pole_gf.moments(2)  # additional high-frequency moment
+    >>> occ_m2 = gt.density_iw(iws, gf_iw, beta=BETA, moments=[1., m2])
+    >>> occ_fit2 = gt.density_iw(iws, gf_iw, beta=BETA, n_fit=1)
+    >>> exact, occ, occ_m2, occ_fit2
+    (0.17858151..., 0.17934437..., 0.17858150..., 0.17858198...)
+    >>> abs(occ - exact), abs(occ_m2 - exact), abs(occ_fit2 - exact)
+    (0.00076286..., 8.18...e-09, 4.72...e-07)
+
+    using more accurate Padé frequencies
+
+    >>> izp, rp = gt.pade_frequencies(100, beta=BETA)
+    >>> gf_izp = pole_gf.eval_z(izp)
+    >>> occ_izp = gt.density_iw(izp, gf_izp, beta=BETA, weights=rp)
+    >>> occ_izp
+    0.17858151...
+    >>> abs(occ_izp - exact) < 1e-14
+    True
+
+    """
+    # add axis for iws, remove it later at occupation
+    moments = np.asanyarray(moments, dtype=np.float_)[..., np.newaxis, :]
+    if n_fit:
+        n_mom = moments.shape[-1]
+        weight = iws.imag**(n_mom+n_fit)
+        mom_gf = pole.PoleGf.from_z(iws, gf_iw[..., newaxis, :], n_pole=n_fit+n_mom,
+                                    moments=moments, width=None, weight=weight)
+    else:
+        mom_gf = pole.PoleGf.from_moments(moments, width=None)
+    delta_gf_iw = gf_iw.real - mom_gf.eval_z(iws).real
+    return 2./beta*np.sum(weights * delta_gf_iw.real, axis=-1) + mom_gf.occ(beta)[..., 0]
+
+
 def density_error(delta_gf_iw, iw_n, noisy=True):
     """Return an estimate for the upper bound of the error in the density.
 
