@@ -56,130 +56,16 @@ Previously defined:
 
 """
 import logging
-from collections import namedtuple
 
 import numpy as np
 from numpy import newaxis
 
 import gftool as gt
-from gftool import linalg
+from gftool.basis.pole import PoleGf
 
 LOGGER = logging.getLogger(__name__)
 
 
-PoleGf = namedtuple('PoleGf', ['resids', 'poles'])
-
-
-def _get_otype(*args):
-    """Determine the resulting type if arrays are broadcasted."""
-    return sum(np.asarray(arg).reshape(-1)[:1] for arg in args).dtype
-
-
-def pole_gf_from_moments(moments) -> PoleGf:
-    """Find pole Green's function matching given `moments`.
-
-    Finds poles and weights for a pole Green's function matching the given
-    high frequency `moments` for large `z`:
-    `g(z) = np.sum(weights / (z - poles)) = moments / z**np.arange(N)`
-
-    Note that for an odd number of moments, the central pole is at `z = 0`,
-    so `g(0)` diverges.
-
-    Parameters
-    ----------
-    moments : (..., N) float array_like
-        Moments of the high-frequency expansion, where
-        `G(z) = moments / z**np.arange(N)` for large `z`.
-
-    Returns
-    -------
-    gf.resids : (..., N) float np.ndarray
-        Residues (or weight) of the poles.
-    gf.poles : (N) float np.ndarray
-        Position of the poles, these are the Chebyshev nodes for degree `N`.
-
-    Notes
-    -----
-    We employ the similarity of the relation betweens the `moments` and
-    the poles and residues with polynomials and the Vandermond matrix.
-    The poles are chooses as Chebyshev nodes, the residues are calculated
-    accordingly.
-
-    """
-    moments = np.asarray(moments)
-    n_mom = moments.shape[-1]
-    if n_mom == 0:  # non-sense case, but return consistent behaviour
-        return PoleGf(resids=moments.copy(), poles=np.array([]))
-    poles = np.cos(.5*np.pi*np.arange(1, 2*n_mom, 2)/n_mom)
-    if n_mom % 2:
-        poles[n_mom//2] = 0.
-    mat = np.polynomial.polynomial.polyvander(poles, deg=poles.size-1).T
-    mat = mat.reshape((1,)*(moments.ndim - 1) + mat.shape)
-    resid = np.linalg.solve(mat, moments)
-    return PoleGf(resids=resid, poles=poles)
-
-
-def pole_gf_from_tau(gf_tau, n_pole, beta, moments=()) -> PoleGf:
-    """Find pole Green's function fitting `gf_tau`.
-
-    Finds poles and weights for a pole Green's function matching the given
-    Green's function `gf_tau`.
-
-    Note that for an odd number of moments, the central pole is at `z = 0`,
-    so the causal Green's function `g(0)` diverges.
-
-    Parameters
-    ----------
-    gf_tau : (..., N_tau) float np.ndarray
-        Imaginary times Green's function which is fitted.
-    n_pole : int
-        Number of poles to fit.
-    beta : float
-        The inverse temperature :math:`beta = 1/k_B T`.
-    moments : (..., N) float array_like
-        Moments of the high-frequency expansion, where
-        `G(z) = moments / z**np.arange(N)` for large `z`.
-
-    Returns
-    -------
-    gf.resids : (..., N) float np.ndarray
-        Residues (or weight) of the poles.
-    gf.poles : (N) float np.ndarray
-        Position of the poles, these are the Chebyshev nodes for degree `N`.
-
-    Raises
-    ------
-    ValueError
-        If more moments are given than poles are fitted (`len(moments) > n_pole`)
-
-    Notes
-    -----
-    We employ the similarity of the relation betweens the `moments` and
-    the poles and residues with polynomials and the Vandermond matrix.
-    The poles are chooses as Chebyshev nodes, the residues are calculated
-    accordingly.
-
-    """
-    poles = np.cos(.5*np.pi*np.arange(1, 2*n_pole, 2)/n_pole)
-    tau = np.linspace(0, beta, num=gf_tau.shape[-1])
-    gf_sp_mat = gt.pole_gf_tau(tau[:, newaxis], poles[:, newaxis], weights=1, beta=beta)
-    moments = np.asarray(moments)
-    otype = _get_otype(gf_tau, moments, poles)
-    if moments.shape[-1] > 0:
-        if moments.shape[-1] > n_pole:
-            raise ValueError("Too many poles given, system is over constrained. "
-                             f"poles: {n_pole}, moments: {moments.shape[-1]}")
-        constrain_mat = np.polynomial.polynomial.polyvander(poles, deg=moments.shape[-1]-1).T
-        _lstsq_ec = np.vectorize(linalg.lstsq_ec, signature='(m,n),(m),(l,n),(l)->(n)',
-                                 otypes=[otype], excluded={'rcond'})
-        resid = _lstsq_ec(gf_sp_mat, gf_tau, constrain_mat, moments)
-    else:
-        _lstsq = np.vectorize(lambda a, b: np.linalg.lstsq(a, b, rcond=None)[0],
-                              signature='(m,n),(m)->(n)', otypes=[otype])
-        resid = _lstsq(gf_sp_mat, gf_tau)
-    return PoleGf(resids=resid, poles=poles)
-
-
 def iw2tau_dft(gf_iw, beta):
     r"""Discrete Fourier transform of the Hermitian Green's function `gf_iw`.
 
@@ -351,7 +237,7 @@ def iw2tau_dft_soft(gf_iw, beta):
     return gf_tau
 
 
-def iw2tau(gf_iw, beta, moments=(1.,), fourier=iw2tau_dft):
+def iw2tau(gf_iw, beta, moments=(1.,), fourier=iw2tau_dft, n_fit=0):
     r"""Discrete Fourier transform of the Hermitian Green's function `gf_iw`.
 
     Fourier transformation of a fermionic Matsubara Green's function to
@@ -371,6 +257,9 @@ def iw2tau(gf_iw, beta, moments=(1.,), fourier=iw2tau_dft):
         High-frequency moments of `gf_iw`.
     fourier : {`iw2tau_dft`, `iw2tau_dft_soft`}, optional
         Back-end to perform the actual Fourier transformation.
+    n_fit : int, optional
+        Number of additionally fitted moments (in fact, `gf_iw` is fitted, not
+        not directly moments).
 
     Returns
     -------
@@ -446,11 +335,17 @@ def iw2tau(gf_iw, beta, moments=(1.,), fourier=iw2tau_dft):
     """
     moments = np.asarray(moments)
     iws = gt.matsubara_frequencies(range(gf_iw.shape[-1]), beta=beta)
-    pole_gf = pole_gf_from_moments(moments[..., newaxis, :])
-    gf_iw = gf_iw - gt.pole_gf_z(iws, poles=pole_gf.poles, weights=pole_gf.resids)
+    # newaxis in pole_gf inserts axis for iws/tau
+    if n_fit:
+        n_mom = moments.shape[-1]
+        pole_gf = PoleGf.from_z(iws, gf_iw[..., newaxis, :], n_pole=n_fit+n_mom,
+                                moments=moments[..., newaxis, :], weight=iws.imag**(n_mom+n_fit))
+    else:
+        pole_gf = PoleGf.from_moments(moments[..., newaxis, :])
+    gf_iw = gf_iw - pole_gf.eval_z(iws)
     gf_tau = fourier(gf_iw, beta=beta)
     tau = np.linspace(0, beta, num=gf_tau.shape[-1])
-    gf_tau += gt.pole_gf_tau(tau, poles=pole_gf.poles, weights=pole_gf.resids, beta=beta)
+    gf_tau += pole_gf.eval_tau(tau, beta=beta)
     return gf_tau
 
 
@@ -802,7 +697,7 @@ def tau2iw_dft(gf_tau, beta):
 def tau2iw_ft_lin(gf_tau, beta):
     r"""Fourier integration of the real Green's function `gf_tau`.
 
-    Fourier transformation of a fermionc imaginary-time Green's function to
+    Fourier transformation of a fermionic imaginary-time Green's function to
     Matsubara domain.
     We assume a real Green's function `gf_tau`, which is the case for
     commutator Green's functions :math:`G_{AB}(τ) = ⟨A(τ)B⟩` with
@@ -887,7 +782,7 @@ def tau2iw_ft_lin(gf_tau, beta):
 
     """
     gf_tau_full_range = np.concatenate((-gf_tau[..., :-1], gf_tau), axis=-1)
-    n_tau = gf_tau_full_range.shape[-1]  # pylint: disable=unsubscriptable-object
+    n_tau = gf_tau_full_range.shape[-1] - 1  # pylint: disable=unsubscriptable-object
     gf_dft = np.fft.ihfft(gf_tau_full_range[..., :-1])
     d_gf_tau = gf_tau_full_range[..., 1:] - gf_tau_full_range[..., :-1]
     d_gf_dft = np.fft.ihfft(d_gf_tau)
@@ -1044,11 +939,11 @@ def tau2iw(gf_tau, beta, n_pole=None, moments=None, fourier=tau2iw_ft_lin):
                            "\n mom: %s, jump: %s", moments[..., 0], m1)
     if n_pole is None:
         n_pole = moments.shape[-1]
-    pole_gf = pole_gf_from_tau(gf_tau, n_pole=n_pole, beta=beta, moments=moments)
-    gf_tau = gf_tau - gt.pole_gf_tau(tau, poles=pole_gf.poles[..., newaxis, :],
-                                     weights=pole_gf.resids[..., newaxis, :], beta=beta)
+    # add additional axis for tau/iws for easy gu-function calling
+    pole_gf = PoleGf.from_tau(gf_tau[..., newaxis, :], n_pole=n_pole, beta=beta,
+                              moments=moments[..., newaxis, :])
+    gf_tau = gf_tau - pole_gf.eval_tau(tau, beta)
     gf_iw = fourier(gf_tau, beta=beta)
     iws = gt.matsubara_frequencies(range(gf_iw.shape[-1]), beta=beta)
-    gf_iw += gt.pole_gf_z(iws, poles=pole_gf.poles[..., newaxis, :],
-                          weights=pole_gf.resids[..., newaxis, :])
+    gf_iw += pole_gf.eval_z(iws)
     return gf_iw
