@@ -83,7 +83,7 @@ def gf_d1_z(z, half_bandwidth):
 
     See Also
     --------
-    gf_z
+    gftool.lattice.bethe.gf_z
 
     """
     z_rel_inv = np.array(half_bandwidth / z, dtype=np.complex256)
@@ -114,7 +114,7 @@ def gf_d2_z(z, half_bandwidth):
 
     See Also
     --------
-    gf_z
+    gftool.lattice.bethe.gf_z
 
     """
     z_rel = np.array(z / half_bandwidth, dtype=np.complex256)
@@ -147,7 +147,7 @@ def hilbert_transform(xi, half_bandwidth):
 
     Returns
     -------
-    hilbert_transfrom : complex ndarray or complex
+    hilbert_transform : complex ndarray or complex
         Hilbert transform of `xi`.
 
     Notes
@@ -159,7 +159,7 @@ def hilbert_transform(xi, half_bandwidth):
 
     See Also
     --------
-    gf_z
+    gftool.lattice.bethe.gf_z
 
     """
     return gf_z(xi, half_bandwidth)
@@ -199,7 +199,7 @@ def dos(eps, half_bandwidth):
     """
     eps_rel = np.asarray(eps / half_bandwidth)
     dos = np.zeros_like(eps_rel)
-    nonzero = abs(eps_rel) < 1
+    nonzero = (abs(eps_rel) < 1) | np.iscomplex(eps)
     dos[nonzero] = 2. / (np.pi*half_bandwidth) * np.sqrt(1 - eps_rel[nonzero]**2)
     return dos
 
@@ -212,7 +212,7 @@ dos_moment_coefficients = {
     4: 0.125,
     6: 0.078125,
     8: 0.0546875,
-    # not sure if results form wolframalpha are accurate enought, calculated them with mpmath
+    # not sure if results form wolframalpha are accurate enough, calculated them with mpmath
     # 10: 0.0410156,
     # 12: 0.0322266,
     # 14: 0.0261841,
@@ -246,12 +246,12 @@ def dos_moment(m, half_bandwidth):
 
     See Also
     --------
-    dos
+    gftool.lattice.bethe.dos
 
     """
     if m % 2:  # odd moments vanish due to symmetry
         return 0
     try:
         return dos_moment_coefficients[m] * half_bandwidth**m
-    except KeyError:
-        raise NotImplementedError('Calculation of arbitrary moments not implemented.')
+    except KeyError as keyerr:
+        raise NotImplementedError('Calculation of arbitrary moments not implemented.') from keyerr
