@@ -22,6 +22,8 @@ References
 import logging
 
 from abc import ABC, abstractmethod
+from typing import Optional as Opt
+from functools import partial
 from itertools import islice
 
 import numpy as np
@@ -32,6 +34,8 @@ from gftool.precision import PRECISE_TYPES as _PRECISE_TYPES
 LOGGER = logging.getLogger(__name__)
 LOGGER.addHandler(logging.NullHandler())
 
+_nan_std = partial(np.nanstd, ddof=1, axis=0)
+
 
 class KindSelector(ABC):
     """Abstract filter class to determine high-frequency behavior of Pade.
@@ -120,29 +124,126 @@ def FilterNegImag(threshold=1e-8):
     See for example the semi-circular spectral function of the Bethe lattice
     with infinite Coordination number as example.
     """
-    def filter_neg_imag(z, pade_iter):
-        r"""Check which pade approximants have a negative imaginary part.
+    def filter_neg_imag(pade_iter):
+        r"""Check which Pade approximants have a negative imaginary part.
 
         Parameters
         ----------
-        z : complex ndarray
-            The inputpoints at which `pade_iter` is calculated.
-        pade_iter : iterable
-            Iterator yielding the Pade approximants of shape
-            (\*approximant.shape, \*z.shape).
+        pade_iter : iterable of (..., N_z) complex np.ndarray
+            Iterator of analytic continuations as generated by `calc_iterator`.
 
         Returns
         -------
-        is_valid : (len(pade_iter), \*approximants.shape) bool ndarray
+        is_valid : (...) bool np.ndarray
             True for all approximants that fulfill `apporximant.imag < threshold`.
 
         """
-        axis = tuple(-np.arange(z.ndim) - 1)  # keep axis not corresponding to z
-        is_valid = np.array([np.all(pade.imag < threshold, axis=axis) for pade in pade_iter])
+        is_valid = np.array([np.all(pade.imag < threshold, axis=-1) for pade in pade_iter])
+        LOGGER.debug("Filter Pades with positive imaginary part (threshold: %s): %s",
+                     threshold, np.count_nonzero(is_valid, axis=0))
         return is_valid
+
     return filter_neg_imag
 
 
+def FilterNegImagNum(abs_num=None, rel_num=None):
+    """Return function to check how bad the imaginary part gets.
+
+    This methods is designed to create `valid_pades` for `Averager`.
+    The imaginary part of retarded Green's functions and self-energies must be
+    negative, this is checked by this filter.
+    All continuations that are *valid* in this sense are kept, the worst invalid
+    are dropped till only `abs_num` remain.
+
+    Warnings
+    --------
+    Only checked for flat inputs.
+
+    """
+    assert abs_num is None or rel_num is None
+    assert abs_num is not None or rel_num is not None
+    def filter_neg_imag_num(pade_iter):
+        badness = np.array([np.max(pade.imag, axis=-1) for pade in pade_iter])
+        abs_num_ = rel_num * badness.shape[0] if abs_num is None else abs_num
+        if abs_num_ >= badness.shape[0]:
+            LOGGER.warning("Skipping filter, not enough Pades (#Pades = %s)",
+                           badness.shape[0])
+            return np.ones_like(badness, dtype=bool)
+
+        keep = np.argsort(badness, axis=0)[:abs_num_]  # abs_num_ best results
+        is_valid = np.zeros_like(badness, dtype=bool)
+        is_valid[keep] = True
+        is_valid[badness <= 0] = True
+        LOGGER.debug("Filter Pades with positive imaginary part (keep best %s): %s",
+                     abs_num_, np.count_nonzero(is_valid, axis=0))
+        assert np.all(np.count_nonzero(is_valid, axis=0) >= abs_num_)
+        return is_valid
+    return filter_neg_imag_num
+
+
+def FilterHighVariance(rel_num: Opt[float] = None, abs_num: Opt[int] = None):
+    """Return function to filter continuations with highest variance.
+
+    Parameters
+    ----------
+    rel_num : float, optional
+        The relative number of continuations to keep.
+    abs_num : int, optional
+        The absolute number of continuations to keep.
+
+    Returns
+    -------
+    filter_high_variance : callable
+        The filter function (pade_iter) -> np.ndarray.
+
+    """
+    assert abs_num is None or rel_num is None
+    assert abs_num is not None or rel_num is not None
+    if rel_num is not None:
+        assert 0. < rel_num < 1.
+    if abs_num is not None:
+        assert abs_num > 0
+    def filter_high_variance(pade_iter):
+        """Remove the continuations with highest variance.
+
+        Parameters
+        ----------
+        pade_iter : iterable of (..., N_z) complex np.ndarray
+            Iterator of analytic continuations as generated by `calc_iterator`.
+
+        Returns
+        -------
+        is_valid : (...) bool np.ndarray
+            Boolean array indicating which continuations to keep.
+
+        """
+        pade = np.array(list(pade_iter))
+        pade_sum = np.sum(pade, axis=0)
+        N_pades = pade.shape[0]
+        # iteratively remove Pades with larges deviation
+        # why iterative?
+        # Awful Pades might give wrong features in average, so it should be corrected
+        if abs_num is None:
+            abs_num_ = int(rel_num*N_pades)
+        bad = []  # isin needs list not set
+        for nn in range(N_pades, abs_num_, -1):
+            diff = nn*pade - pade_sum  # 50% of time
+            distance = np.sum(diff.real**2 + diff.imag**2, axis=-1)  # 40% of time
+            badness = np.argsort(distance, axis=0)[::-1]  # truncate what is not needed
+            newbad = badness[np.isin(badness, bad, invert=True)][0]
+            bad.append(newbad)
+            pade_sum -= pade[newbad]
+        try:
+            is_valid = np.ones_like(distance, dtype=bool)
+        except UnboundLocalError:
+            LOGGER.warning("Not enough pades to filter (#Pades = %s)", N_pades)
+            return np.ones_like(pade[..., 0], dtype=bool)
+        is_valid[badness[:-abs_num_]] = False
+        # assert set(badness[:-abs_num_]) == set(bad)  # FIXME
+        return is_valid
+    return filter_high_variance
+
+
 def _contains_nan(array) -> bool:
     """Check if `array` contains any NaN."""
     flat = array.reshape(-1)
@@ -191,6 +292,7 @@ def coefficients(z, fct_z) -> np.ndarray:
     return res.astype(complex_pres, copy=False)
 
 
+@partial(np.vectorize, otypes=[np.complex], signature='(n),(n)->(n)')
 def masked_coefficients(z, fct_z):
     """Calculate coefficients but ignore extreme values.
 
@@ -226,15 +328,14 @@ def masked_coefficients(z, fct_z):
                 if comparable_mag(mat[last_it, jj]):
                     mat_pi[jj] = (last_coeff/mat[last_it, jj] - 1.)/(z[jj] - z[last_it])
                 elif abs(last_coeff) < abs(mat[last_it, jj])*cutoff:  # tiny quotient
-                    print('truncate')
                     mat_pi[jj] = (-1)/(z[jj] - z[last_it])
                 else:  # huge quotient
-                    print('infty')
                     mat_pi[jj] = np.infty
             last_it = ii
             last_coeff = mat_pi[ii]
         else:
             mask[ii] = False
+    # pylint: disable=invalid-unary-operand-type
     LOGGER.info("Number of eliminated coefficients: %s", np.count_nonzero(~mask))
     return mat.diagonal(axis1=0, axis2=-1)
 
@@ -308,7 +409,7 @@ def Averager(z_in, coeff, *, valid_pades, kind: KindSelector):
         evaluates to false will be ignored for the average.
     kind : {KindGf, KindSelf}
         Defines the asymptotic of the continued function and the number of
-        minumum and maximum input points used for Pade. For `KindGf` the
+        minimum and maximum input points used for Pade. For `KindGf` the
         function goes like :math:`1/z` for large `z`, for `KindSelf` the
         function behaves like a constant for large `z`.
 
@@ -380,7 +481,7 @@ def Averager(z_in, coeff, *, valid_pades, kind: KindSelector):
             pades[~valid_pades] = np.nan + 1j*np.nan
 
         pade_avg = np.nanmean(pades, axis=0)
-        std = np.nanstd(pades.real, axis=0, ddof=1) + 1j*np.nanstd(pades.imag, axis=0, ddof=1)
+        std = _nan_std(pades.real) + 1j*_nan_std(pades.imag)
 
         return Result(x=pade_avg, err=std)
     return average
@@ -495,15 +596,50 @@ def Mod_Averager(z_in, coeff, mod_fct, *, valid_pades, kind: KindSelector, vecto
         pade_avg = np.nanmean(mod_pade, axis=0)
         # define helper pade_std np.nanstd( ,axis=0, ddof=1) if complex...
         if np.iscomplexobj(mod_pade):
-            std = np.nanstd(mod_pade.real, axis=0, ddof=1) + 1j*np.nanstd(mod_pade.imag, axis=0, ddof=1)
+            std = _nan_std(mod_pade.real) + 1j*_nan_std(mod_pade.imag)
         else:
-            std = np.nanstd(mod_pade, axis=0, ddof=1)
+            std = _nan_std(mod_pade)
 
         return Result(x=pade_avg, err=std)
     mod_average.__doc__ = mod_average.__doc__.format(mod_fct=mod_fct)
     return mod_average
 
 
+def apply_filter(*filters, validity_iter):
+    r"""Handle usage of filters for Pade.
+
+    Parameters
+    ----------
+    filters : callable
+        Functions to determine which continuations to keep.
+    validity_iter : iterable of (..., N_z) complex np.ndarray
+        The iterable of analytic continuations as generated by `calc_iterator`.
+
+    Returns
+    -------
+    is_valid : (...) bool np.ndarray
+        Array to index which continuations are good.
+
+    """
+    if len(filters) == 1:
+        return filters[0](validity_iter)
+    validity_iter = np.array(list(validity_iter))
+    shape = validity_iter.shape
+    validity_iter = np.moveaxis(validity_iter, 0, -2).reshape((-1, shape[0], shape[-1]))
+    is_valid = np.ones(validity_iter.shape[0:2], dtype=bool)
+    for i_valid, i_validity_iter in zip(is_valid, validity_iter):
+        for filt in filters:
+            is_valid_filt = filt(i_validity_iter[i_valid])
+            i_valid[i_valid] = is_valid_filt
+            if np.count_nonzero(i_valid) == 0:
+                raise RuntimeError(
+                    f"No Pade is valid due to filter {filt}.\n"
+                    f"No solution found for coefficient (shape: {validity_iter.shape[1:-1]}) axes "
+                    f"{np.argwhere(~is_valid.any(axis=0))}"
+                )
+    return np.moveaxis(is_valid, 0, -1).reshape(shape[:-1])
+
+
 def averaged(z_out, z_in, *, valid_z=None, fct_z=None, coeff=None,
              filter_valid=None, kind: KindSelector):
     """Return the averaged Pade continuation with its variance.
@@ -529,15 +665,15 @@ def averaged(z_out, z_in, *, valid_z=None, fct_z=None, coeff=None,
     coeff : (N_in,) complex ndarray, optional
         Coefficients for Pade, calculated from `pade.coefficients`. Can be given
         instead of `fct_z`.
-    filter_valid : callable
+    filter_valid : callable or iterable of callable
         Function determining which approximants to keep. The signature should
-        be filter_valid(ndarray, iterable) -> bool ndarray.
-        Currently there are the functions {`FilterNegImag`, } implemented
-        to generate filter functions. Look into the implemented for details
-        to create new filters.
+        be filter_valid(iterable) -> bool ndarray.
+        Currently there are the functions {`FilterNegImag`, `FilterNegImagNum`,
+        `FilterHighVariance`} implemented to generate filter functions.
+        Look into the implemented for details to create new filters.
     kind : {KindGf, KindSelf}
         Defines the asymptotic of the continued function and the number of
-        minumum and maximum input points used for Pade. For `KindGf` the
+        minimum and maximum input points used for Pade. For `KindGf` the
         function goes like :math:`1/z` for large `z`, for `KindSelf` the
         function behaves like a constant for large `z`.
 
@@ -550,13 +686,19 @@ def averaged(z_out, z_in, *, valid_z=None, fct_z=None, coeff=None,
 
     """
     assert fct_z is None or coeff is None
-    if fct_z is not None:
-        coeff = coefficients(z_in, fct_z=fct_z)
+    z_in = z_in[:kind.stop]
+    coeff = (coefficients(z_in, fct_z=fct_z[..., :kind.stop]) if coeff is None
+             else coeff[..., :kind.stop])
     if valid_z is None:
         valid_z = z_out
+    valid_z = valid_z.reshape(-1)
     if filter_valid is not None:
         validity_iter = kind.islice(calc_iterator(valid_z, z_in, coeff=coeff))
-        is_valid = filter_valid(valid_z, validity_iter)
+        try:
+            filters = tuple(filter_valid)
+        except TypeError:  # only one filter given
+            filters = (filter_valid,)
+        is_valid = apply_filter(*filters, validity_iter=validity_iter)
     else:
         is_valid = np.ones((len(kind), *coeff.shape[:-1]), dtype=bool)
 
